@@ -38,7 +38,7 @@ export
 
 using LinearAlgebra, SparseArrays
 using StaticArrays, WoodburyMatrices, Ratios, AxisAlgorithms, OffsetArrays
-using ChainRulesCore, Requires
+using ChainRulesCore
 
 using Base: @propagate_inbounds, HasEltype, EltypeUnknown, HasLength, IsInfinite,
     SizeUnknown, Indices
@@ -94,6 +94,7 @@ struct Throw{GT<:Union{GridType,Nothing}} <: BoundaryCondition gt::GT end
 struct Flat{GT<:Union{GridType,Nothing}} <: BoundaryCondition gt::GT end
 "`Line(gt)` uses a constant slope for extrapolation"
 struct Line{GT<:Union{GridType,Nothing}} <: BoundaryCondition gt::GT end
+"`Free(gt)` the free boundary condition makes sure the interpoland has a continuous third derivative at the second-to-outermost cell boundary"
 struct Free{GT<:Union{GridType,Nothing}} <: BoundaryCondition gt::GT end
 "`Periodic(gt)` applies periodic boundary conditions"
 struct Periodic{GT<:Union{GridType,Nothing}} <: BoundaryCondition gt::GT end
@@ -108,6 +109,12 @@ struct InPlaceQ{GT<:Union{GridType,Nothing}} <: BoundaryCondition gt::GT end
 const Natural = Line
 
 (::Type{BC})() where BC<:BoundaryCondition = BC(nothing)
+for BC in (:Throw, :Flat, :Line, :Free, :Reflect, :InPlace, :InPlaceQ, :Periodic)
+    eval(quote
+        $BC(::Type{GT}) where GT <: GridType = $BC{GT}(GT())
+        $BC{GT}(::Type{GT}) where GT <: GridType = $BC{GT}(GT())
+    end)
+end
 function Base.show(io::IO, bc::BoundaryCondition)
     print(io, nameof(typeof(bc)), '(')
     bc.gt === nothing || show(io, bc.gt)
@@ -277,25 +284,34 @@ struct InterpGetindex{N,A<:AbstractArray{<:Any,N}}
     InterpGetindex(A::AbstractArray) = new{ndims(A),typeof(A)}(A)
 end
 @inline Base.getindex(A::InterpGetindex{N}, I::Vararg{Union{Int,WeightedIndex},N}) where {N} =
-    interp_getindex(A.coeffs, ntuple(_ -> 0, Val(N)), map(indexflag, I)...)
-indexflag(I::Int) = I
-@inline indexflag(I::WeightedIndex) = indextuple(I), weights(I)
+    interp_getindex(A.coeffs, ntuple(zero, Val(N)), map(indexflag, I)...)
+@inline indexflag(I) = indextuple(I), weights(I)
+
+# Direct recursion would allow more eager inference before julia 1.11.
+# Normalize all index into the same format.
+struct One end  # Singleton for express weights of no-interp dims
+indextuple(I::Int) = (I,)
+weights(::Int) = (One(),)
+
+struct Zero end # Singleton for dim expansion termination
 
 # A recursion-based `interp_getindex`, which follows a "move processed indexes to the back" strategy
 # `I` contains the processed index, and (wi1, wis...) contains the yet-to-be-processed indexes
-# Here we meet a no-interp dim, just append the index to `I`'s end.
-@inline interp_getindex(A, I, wi1::Int, wis...) =
-    interp_getindex(A, (Base.tail(I)..., wi1), wis...)
 # Here we handle the expansion of a single dimension.
-@inline interp_getindex(A, I, wi1::NTuple{2,Tuple{Any,Vararg{Any,N}}}, wis...) where {N} =
-    wi1[2][end] * interp_getindex(A, (Base.tail(I)..., wi1[1][end]), wis...) +
-    interp_getindex(A, I, map(Base.front, wi1), wis...)
-@inline interp_getindex(A, I, wi1::NTuple{2,Tuple{Any}}, wis...) =
-    wi1[2][1] * interp_getindex(A, (Base.tail(I)..., wi1[1][1]), wis...)
+@inline function interp_getindex(A, I, (is, ws)::NTuple{2,Tuple}, wis...)
+    itped1 = interp_getindex(A, (Base.tail(I)..., is[end]), wis...)
+    witped = interp_getindex(A, I, (Base.front(is), Base.front(ws)), wis...)
+    _weight_itp(ws[end], itped1, witped)
+end
+interp_getindex(_, _, ::NTuple{2,Tuple{}}, ::Vararg) = Zero()
 # Termination
 @inline interp_getindex(A::AbstractArray{T,N}, I::Dims{N}) where {T,N} =
     @inbounds A[I...] # all bounds-checks have already happened
 
+_weight_itp(w, i, wir) = w * i + wir
+_weight_itp(::One, i, ::Zero) = i
+_weight_itp(w, i, ::Zero) = w * i
+
 """
     w = value_weights(degree, δx)
 
@@ -417,18 +433,20 @@ _checkbounds(::CheckWillPass, itp, x) = true
 _checkbounds(::NeedsCheck, itp, x) = checklubounds(lbounds(itp), ubounds(itp), x)
 
 checklubounds(ls, us, xs) = _checklubounds(true, ls, us, xs)
-_checklubounds(tf::Bool, ls, us, xs::Tuple{Number, Vararg{Any}}) =
-    _checklubounds(tf & (ls[1] <= xs[1] <= us[1]), Base.tail(ls), Base.tail(us), Base.tail(xs))
-_checklubounds(tf::Bool, ls, us, xs::Tuple{AbstractVector, Vararg{Any}}) =
+_checklubounds(tf::Bool, ls::Tuple, us::Tuple, xs::Tuple) =
     _checklubounds(tf & allbetween(ls[1], xs[1], us[1]), Base.tail(ls), Base.tail(us), Base.tail(xs))
+_checklubounds(tf::Bool, ::Tuple{}, ::Tuple{}, xs::Tuple) =
+    _checklubounds(tf & all(isone, xs[1]), (), (), Base.tail(xs))
+_checklubounds(tf::Bool, ls::Tuple, us::Tuple, ::Tuple{}) =
+    _checklubounds(tf & (ls[1] == us[1]), Base.tail(ls), Base.tail(us), ())
 _checklubounds(tf::Bool, ::Tuple{}, ::Tuple{}, ::Tuple{}) = tf
 
 maybe_clamp(itp, xs) = maybe_clamp(BoundsCheckStyle(itp), itp, xs)
 maybe_clamp(::NeedsCheck, itp, xs) = map(clamp, xs, lbounds(itp), ubounds(itp))
 maybe_clamp(::CheckWillPass, itp, xs) = xs
 
-Base.hash(x::AbstractInterpolation, h::UInt = zero(UInt)) = Base.hash_uint(3h - objectid(x))
-Base.hash(x::AbstractExtrapolation, h::UInt = zero(UInt)) = Base.hash_uint(3h - objectid(x))
+Base.hash(x::AbstractInterpolation, h::UInt) = Base.hash_uint(3h - objectid(x))
+Base.hash(x::AbstractExtrapolation, h::UInt) = Base.hash_uint(3h - objectid(x))
 
 include("nointerp/nointerp.jl")
 include("b-splines/b-splines.jl")
@@ -449,8 +467,14 @@ if VERSION >= v"1.6"
     include("gpu_support.jl")
 end
 
+if !isdefined(Base, :get_extension)
+using Requires
+end
+
+@static if !isdefined(Base, :get_extension)
 function __init__()
-    @require Unitful="1986cc42-f94f-5a68-af5c-568840ba703d" include("requires/unitful.jl")
+    @require Unitful="1986cc42-f94f-5a68-af5c-568840ba703d" include("../ext/InterpolationsUnitfulExt.jl")
+end
 end
 
 end # module
